'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _cloneDeep = require('lodash/cloneDeep');

var _cloneDeep2 = _interopRequireDefault(_cloneDeep);

var _find2 = require('lodash/find');

var _find3 = _interopRequireDefault(_find2);

var _omit = require('lodash/omit');

var _omit2 = _interopRequireDefault(_omit);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var pack = function pack() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var _ref$idField = _ref.idField;
  var idField = _ref$idField === undefined ? 'id' : _ref$idField;
  var _ref$parentField = _ref.parentField;
  var parentField = _ref$parentField === undefined ? 'parent' : _ref$parentField;
  var _ref$childrenField = _ref.childrenField;
  var childrenField = _ref$childrenField === undefined ? 'children' : _ref$childrenField;
  return function (rows) {
    if (!Array.isArray(rows)) {
      return [];
    }

    if (!rows.length) {
      return rows;
    }

    var ret = [];
    var previousParents = void 0;

    // Clone deep to guarantee immutability (implementation relies on
    // references)
    (0, _cloneDeep2.default)(rows).forEach(function (row) {
      var rowParent = row[parentField];

      if (typeof rowParent !== 'undefined' && rowParent !== null) {
        // Check if the parent field matches the accumulated potential parents
        var foundParent = (0, _find3.default)(previousParents, _defineProperty({}, idField, rowParent));

        if (foundParent) {
          // Found the matching parent, set up children + push
          if (!foundParent[childrenField]) {
            foundParent[childrenField] = [];
          }

          foundParent[childrenField].push((0, _omit2.default)(row, parentField));

          previousParents.push(last(foundParent[childrenField]));
        } else {
          console.warn('Invalid parent id', row, previousParents); // eslint-disable-line no-console
        }
      } else {
        ret.push(row);

        // No parents, flush lookup
        previousParents = [row];
      }
    });

    return ret;
  };
};

function last(arr) {
  return arr[arr.length - 1];
}

exports.default = pack;